<?php
namespace boardtalk\smartpreview\controller;

use phpbb\template\template;
use phpbb\controller\helper;
use phpbb\db\driver\driver_interface;
use phpbb\user;

class main_controller
{
    protected $template;
    protected $helper;
    protected $db;
    protected $user;

    public function __construct(template $template, helper $helper, driver_interface $db, user $user)
    {
        $this->template = $template;
        $this->helper   = $helper;
        $this->db       = $db;
        $this->user     = $user;
    }

    public function handle()
    {
        // ── Load text‐display helpers ──
        global $phpbb_root_path, $phpEx;
        if (!function_exists('generate_text_for_display'))
        {
            include_once $phpbb_root_path . 'includes/functions_display.' . $phpEx;
        }

        $user_id = (int) $this->user->data['user_id'];

        // 1) Fetch latest topics
        $limit = 15;
        $sql = 'SELECT t.topic_id, t.topic_title,
                       t.topic_first_post_id, t.topic_last_post_id,
                       t.topic_views
                FROM ' . TOPICS_TABLE . ' t
                WHERE t.topic_visibility = 1
                ORDER BY t.topic_last_post_time DESC';
        $res = $this->db->sql_query_limit($sql, $limit);

        $topics   = [];
        $post_ids = [];
        while ($row = $this->db->sql_fetchrow($res))
        {
            $tid = (int) $row['topic_id'];
            $topics[$tid] = [
                'title'       => $row['topic_title'],
                'first_id'    => (int) $row['topic_first_post_id'],
                'last_id'     => (int) $row['topic_last_post_id'],
                'views'       => (int) $row['topic_views'],
                'replies'     => 0,

                // first‐post placeholders
                'first_user'            => '',
                'first_text'            => '',
                'first_time'            => 0,
                'first_avatar'          => '',
                'first_bbcode_uid'      => '',
                'first_bbcode_bitfield' => '',
                'first_enable_bbcode'   => 0,
                'first_enable_smilies'  => 0,
                'first_enable_magic_url'=> 0,

                // last‐post placeholders
                'last_user'             => '',
                'last_text'             => '',
                'last_time'             => 0,
                'last_bbcode_uid'       => '',
                'last_bbcode_bitfield'  => '',
                'last_enable_bbcode'    => 0,
                'last_enable_smilies'   => 0,
                'last_enable_magic_url' => 0,
            ];

            $post_ids[] = $row['topic_first_post_id'];
            if ($row['topic_last_post_id'] !== $row['topic_first_post_id'])
            {
                $post_ids[] = $row['topic_last_post_id'];
            }
        }
        $this->db->sql_freeresult($res);

        // 2) Count replies per topic
        if (!empty($topics))
        {
            $topic_ids = array_keys($topics);
            $sql = 'SELECT topic_id, COUNT(post_id) AS total_posts
                    FROM ' . POSTS_TABLE . '
                    WHERE ' . $this->db->sql_in_set('topic_id', $topic_ids) . '
                    GROUP BY topic_id';
            $r2 = $this->db->sql_query($sql);
            while ($row2 = $this->db->sql_fetchrow($r2))
            {
                $tid2 = (int) $row2['topic_id'];
                $topics[$tid2]['replies'] = max(0, (int) $row2['total_posts'] - 1);
            }
            $this->db->sql_freeresult($r2);
        }

		// 2.5) Count new replies per topic based on each topic’s mark_time
		foreach ($topics as $tid => &$t)
		{
			if ($user_id !== ANONYMOUS && $t['replies'] > 0)
			{
				// 1) Get this topic’s mark_time
				$sql = 'SELECT mark_time
						FROM ' . TOPICS_TRACK_TABLE . '
						WHERE user_id   = ' . $user_id . '
						  AND topic_id  = ' . $tid;
				$r = $this->db->sql_query($sql);
				if ($row = $this->db->sql_fetchrow($r))
				{
					$mark_time = (int) $row['mark_time'];
				}
				else
				{
					// never marked, fall back to lastvisit
					$mark_time = (int) $this->user->data['user_lastvisit'];
				}
				$this->db->sql_freeresult($r);

				// 2) Count all replies after that time (exclude the first post)
				$sql = 'SELECT COUNT(post_id) AS new_posts
						FROM ' . POSTS_TABLE . '
						WHERE topic_id  = ' . $tid . '
						  AND post_time > ' . $mark_time . '
						  AND post_id   <> ' . $t['first_id'];
				$r2 = $this->db->sql_query($sql);
				$t['new_replies'] = (int) $this->db->sql_fetchfield('new_posts');
				$this->db->sql_freeresult($r2);
			}
			else
			{
				$t['new_replies'] = 0;
			}
		}
		unset($t);


        // 3) Fetch first & last posts + avatars + parser flags
        if (!empty($post_ids))
        {
            $ids_sql = $this->db->sql_in_set('p.post_id', array_unique($post_ids));
            $sql = 'SELECT p.post_id, p.post_text, p.post_time,
                           p.bbcode_uid, p.bbcode_bitfield,
                           p.enable_bbcode, p.enable_smilies, p.enable_magic_url,
                           u.username, u.user_avatar, u.user_avatar_type,
                           u.user_avatar_width, u.user_avatar_height
                    FROM ' . POSTS_TABLE . ' p
                    LEFT JOIN ' . USERS_TABLE . ' u
                      ON p.poster_id = u.user_id
                    WHERE ' . $ids_sql;
            $r = $this->db->sql_query($sql);

            while ($row = $this->db->sql_fetchrow($r))
            {
                $pid = (int) $row['post_id'];
                foreach ($topics as $tid => &$t)
                {
                    if ($pid === $t['first_id'])
                    {
                        $t['first_user']            = $row['username'];
                        $t['first_text']            = $row['post_text'];
                        $t['first_time']            = (int) $row['post_time'];
                        $t['first_bbcode_uid']      = $row['bbcode_uid'];
                        $t['first_bbcode_bitfield'] = $row['bbcode_bitfield'];
                        $t['first_enable_bbcode']   = (int) $row['enable_bbcode'];
                        $t['first_enable_smilies']  = (int) $row['enable_smilies'];
                        $t['first_enable_magic_url']= (int) $row['enable_magic_url'];
                        $t['first_avatar'] = get_user_avatar(
                            $row['user_avatar'],
                            $row['user_avatar_type'],
                            $row['user_avatar_width'],
                            $row['user_avatar_height'],
                            $row['username']
                        );
                    }
                    if ($pid === $t['last_id'])
                    {
                        $t['last_user']             = $row['username'];
                        $t['last_text']             = $row['post_text'];
                        $t['last_time']             = (int) $row['post_time'];
                        $t['last_bbcode_uid']       = $row['bbcode_uid'];
                        $t['last_bbcode_bitfield']  = $row['bbcode_bitfield'];
                        $t['last_enable_bbcode']    = (int) $row['enable_bbcode'];
                        $t['last_enable_smilies']   = (int) $row['enable_smilies'];
                        $t['last_enable_magic_url'] = (int) $row['enable_magic_url'];
                    }
                }
                unset($t);
            }
            $this->db->sql_freeresult($r);
        }

        // 4) Determine which reply to show (first unread or last)
        foreach ($topics as $tid => &$t)
        {
            $t['reply_exists']           = false;
            $t['reply_label']            = '';
            $t['reply_id']               = 0;
            $t['reply_user']             = '';
            $t['reply_time']             = 0;
            $t['reply_text']             = '';
            $t['reply_bbcode_uid']       = '';
            $t['reply_bbcode_bitfield']  = '';
            $t['reply_enable_bbcode']    = 0;
            $t['reply_enable_smilies']   = 0;
            $t['reply_enable_magic_url'] = 0;

            if ($t['replies'] > 0)
            {
                $found_unread = false;

                if ($user_id !== ANONYMOUS)
                {
                    // get topic mark time
                    $sql = 'SELECT mark_time
                            FROM ' . TOPICS_TRACK_TABLE . "
                            WHERE user_id = $user_id
                              AND topic_id = $tid";
                    $r = $this->db->sql_query($sql);
                    if ($row = $this->db->sql_fetchrow($r))
                    {
                        $mark_time = (int) $row['mark_time'];
                    }
                    else
                    {
                        $mark_time = (int) $this->user->data['user_lastvisit'];
                    }
                    $this->db->sql_freeresult($r);

                    // fetch first post _after_ mark_time
                    $sql = 'SELECT p.post_id, p.post_text, p.post_time,
                                   p.bbcode_uid, p.bbcode_bitfield,
                                   p.enable_bbcode, p.enable_smilies, p.enable_magic_url,
                                   u.username
                            FROM ' . POSTS_TABLE . " p
                            LEFT JOIN " . USERS_TABLE . " u
                              ON p.poster_id = u.user_id
                            WHERE p.topic_id = $tid
                              AND p.post_time > $mark_time
                              AND p.post_id <> {$t['first_id']}
                            ORDER BY p.post_time ASC";
                    $r2 = $this->db->sql_query_limit($sql, 1);
                    if ($row2 = $this->db->sql_fetchrow($r2))
                    {
                        $found_unread               = true;
                        $t['reply_id']               = (int) $row2['post_id'];
                        $t['reply_user']             = $row2['username'];
                        $t['reply_time']             = (int) $row2['post_time'];
                        $t['reply_text']             = $row2['post_text'];
                        $t['reply_bbcode_uid']       = $row2['bbcode_uid'];
                        $t['reply_bbcode_bitfield']  = $row2['bbcode_bitfield'];
                        $t['reply_enable_bbcode']    = (int) $row2['enable_bbcode'];
                        $t['reply_enable_smilies']   = (int) $row2['enable_smilies'];
                        $t['reply_enable_magic_url'] = (int) $row2['enable_magic_url'];
                        $t['reply_label']            = ' First unread reply';
                    }
                    $this->db->sql_freeresult($r2);
                }

                if (!$found_unread)
                {
                    // fallback to last reply
                    $t['reply_id']               = $t['last_id'];
                    $t['reply_user']             = $t['last_user'];
                    $t['reply_time']             = $t['last_time'];
                    $t['reply_text']             = $t['last_text'];
                    $t['reply_bbcode_uid']       = $t['last_bbcode_uid'];
                    $t['reply_bbcode_bitfield']  = $t['last_bbcode_bitfield'];
                    $t['reply_enable_bbcode']    = $t['last_enable_bbcode'];
                    $t['reply_enable_smilies']   = $t['last_enable_smilies'];
                    $t['reply_enable_magic_url'] = $t['last_enable_magic_url'];
                    $t['reply_label']            = '';
                }

                $t['reply_exists'] = true;
            }
        }
        unset($t);

        // 5) Parse, truncate (30 words), and assign everything to template
        foreach ($topics as $tid => $t)
        {
            // ── FIRST post ──
            $opts1 = 0;
            if ($t['first_enable_bbcode'])    { $opts1 |= OPTION_FLAG_BBCODE; }
            if ($t['first_enable_smilies'])   { $opts1 |= OPTION_FLAG_SMILIES; }
            if ($t['first_enable_magic_url']) { $opts1 |= OPTION_FLAG_LINKS; }

            $parsed1 = generate_text_for_display(
                $t['first_text'],
                $t['first_bbcode_uid'],
                $t['first_bbcode_bitfield'],
                $opts1
            );
            $words1      = preg_split('/\s+/', strip_tags($parsed1), -1, PREG_SPLIT_NO_EMPTY);
            $first_trunc = count($words1) > 30;
            $first_excerpt = $first_trunc
                           ? implode(' ', array_slice($words1, 0, 30)) . '...'
                           : $parsed1;

            // ── REPLY ──
            $reply_excerpt = '';
            $reply_trunc   = false;
            if ($t['reply_exists'])
            {
                $opts2 = 0;
                if ($t['reply_enable_bbcode'])    { $opts2 |= OPTION_FLAG_BBCODE; }
                if ($t['reply_enable_smilies'])   { $opts2 |= OPTION_FLAG_SMILIES; }
                if ($t['reply_enable_magic_url']) { $opts2 |= OPTION_FLAG_LINKS; }

                $parsed2 = generate_text_for_display(
                    $t['reply_text'],
                    $t['reply_bbcode_uid'],
                    $t['reply_bbcode_bitfield'],
                    $opts2
                );
                $words2     = preg_split('/\s+/', strip_tags($parsed2), -1, PREG_SPLIT_NO_EMPTY);
                $reply_trunc = count($words2) > 30;
                $reply_excerpt = $reply_trunc
                               ? implode(' ', array_slice($words2, 0, 30)) . '...'
                               : $parsed2;
            }

            // new‐replies count
            $new_replies = isset($t['new_replies']) ? $t['new_replies'] : 0;

            $this->template->assign_block_vars('topics', [
                // avatar
                'HAS_AVATAR'         => (bool) $t['first_avatar'],
                'AVATAR'             => $t['first_avatar'],

                // first‐post info
                'TOPIC_TITLE'        => $t['title'],
                'U_FIRST_POST'       => append_sid('viewtopic.php', 't=' . $tid),
                'FIRST_POSTER_FULL'  => $t['first_user'],
                'FIRST_POST_TIME'    => date('j M Y, H:i', $t['first_time']),
                'FIRST_POST_EXCERPT' => $first_excerpt,
                'FIRST_TRUNCATED'    => (int) $first_trunc,

                // stats
                'TOPIC_VIEWS'        => $t['views'],
                'TOPIC_REPLIES'      => $t['replies'],
				'TOPIC_NEW_REPLIES'	 => $t['new_replies'],

                // reply info
                'HAS_REPLY'          => (int) $t['reply_exists'],
                'REPLY_LABEL'        => $t['reply_label'],
                'U_REPLY_POST'       => append_sid('viewtopic.php', 'p=' . $t['reply_id']) . '#p' . $t['reply_id'],
                'REPLY_POSTER'       => $t['reply_user'],
                'REPLY_TIME'         => date('j M Y, H:i', $t['reply_time']),
                'REPLY_EXCERPT'      => $reply_excerpt,
                'REPLY_TRUNCATED'    => (int) $reply_trunc,
            ]);
        }

        return $this->helper->render(
            'recent_body.html',
            $this->user->lang('SMARTPREVIEW_TITLE')
        );
    }
}
